"""
Contains a list of EGL functions to generate dispatch functions for.

This is used from gen_egl_dispatch.py.

EGL_FUNCTIONS is a sequence of (name, eglData) pairs, where name is the name
of the function, and eglData is a dictionary containing data about that
function.

The values in the eglData dictionary are:
- method (string):
    How to select a vendor library. See "Method values" below.

- prefix (string):
    This string is prepended to the name of the dispatch function. If
    unspecified, the default is "" (an empty string).

- static (boolean)
  If True, this function should be declared static.

- "public" (boolean)
    If True, the function should be exported from the library. Vendor libraries
    generally should not use this.

- extension (string):
    If specified, this is the name of a macro to check for before defining a
    function. Used for checking for extension macros and such.

- retval (string):
    If specified, this is a C expression with the default value to return if we
    can't find a function to call. By default, it will try to guess from the
    return type: EGL_NO_whatever for the various handle types, NULL for
    pointers, and zero for everything else.

method values:
- "custom"
    The dispatch stub will be hand-written instead of generated.

- "none"
    No dispatch function exists at all, but the function should still have an
    entry in the index array. This is for other functions that a stub may need
    to call that are implemented in libEGL itself.

- "display"
    Select a vendor from an EGLDisplay argument.

- "device"
    Select a vendor from an EGLDeviceEXT argument.

- "current"
    Select the vendor that owns the current context.
"""

def _eglFunc(name, method, inheader, static=False, public=False, prefix="", extension=None, retval=None):
    """
    A convenience function to define an entry in the EGL function list.
    """
    values = {
        "method" : method,
        "prefix" : prefix,
        "extension" : extension,
        "retval" : retval,
        "static" : static,
        "public" : public,
        "inheader" : inheader,
    }
    return (name, values)

def _eglCore(name, method, **kwargs):
    return _eglFunc(name, method, public=True, inheader=False, **kwargs)

def _eglExt(name, method, static=None, **kwargs):
    if (static is None):
        static = (method != "custom")
    inheader = not static
    return _eglFunc(name, method, static=static, inheader=inheader, public=False, **kwargs)

EGL_FUNCTIONS = (
    # EGL_VERSION_1_0
    _eglCore("eglChooseConfig",                      "display"),
    _eglCore("eglCopyBuffers",                       "display"),
    _eglCore("eglCreateContext",                     "display"),
    _eglCore("eglCreatePbufferSurface",              "display"),
    _eglCore("eglCreatePixmapSurface",               "display"),
    _eglCore("eglCreateWindowSurface",               "display"),
    _eglCore("eglDestroyContext",                    "display"),
    _eglCore("eglDestroySurface",                    "display"),
    _eglCore("eglGetConfigAttrib",                   "display"),
    _eglCore("eglGetConfigs",                        "display"),
    _eglCore("eglQueryContext",                      "display"),
    _eglCore("eglQuerySurface",                      "display"),
    _eglCore("eglSwapBuffers",                       "display"),
    _eglCore("eglWaitGL",                            "current", retval="EGL_TRUE"),
    _eglCore("eglWaitNative",                        "current", retval="EGL_TRUE"),
    _eglCore("eglTerminate",                         "display"),
    _eglCore("eglInitialize",                        "display"),

    _eglCore("eglGetCurrentDisplay",                 "custom"),
    _eglCore("eglGetCurrentSurface",                 "custom"),
    _eglCore("eglGetDisplay",                        "custom"),
    _eglCore("eglGetError",                          "custom"),
    _eglCore("eglGetProcAddress",                    "custom"),
    _eglCore("eglMakeCurrent",                       "custom"),
    _eglCore("eglQueryString",                       "custom"),

    # EGL_VERSION_1_1
    _eglCore("eglBindTexImage",                      "display"),
    _eglCore("eglReleaseTexImage",                   "display"),
    _eglCore("eglSurfaceAttrib",                     "display"),
    _eglCore("eglSwapInterval",                      "display"),

    # EGL_VERSION_1_2
    _eglCore("eglCreatePbufferFromClientBuffer",     "display"),
    _eglCore("eglWaitClient",                        "current", retval="EGL_TRUE"),
    _eglCore("eglBindAPI",                           "custom"),
    _eglCore("eglQueryAPI",                          "custom"),
    _eglCore("eglReleaseThread",                     "custom"),

    # EGL_VERSION_1_4
    _eglCore("eglGetCurrentContext",                 "custom"),

    # EGL_VERSION_1_5
    _eglCore("eglCreateSync",                        "display"),
    _eglCore("eglDestroySync",                       "display"),
    _eglCore("eglClientWaitSync",                    "display"),
    _eglCore("eglGetSyncAttrib",                     "display"),
    _eglCore("eglCreateImage",                       "display"),
    _eglCore("eglDestroyImage",                      "display"),
    _eglCore("eglCreatePlatformWindowSurface",       "display"),
    _eglCore("eglCreatePlatformPixmapSurface",       "display"),
    _eglCore("eglWaitSync",                          "display"),
    _eglCore("eglGetPlatformDisplay",                "custom"),

    # EGL_EXT_platform_base
    _eglExt("eglCreatePlatformWindowSurfaceEXT",    "display"),
    _eglExt("eglCreatePlatformPixmapSurfaceEXT",    "display"),
    _eglExt("eglGetPlatformDisplayEXT",             "custom"),

    # EGL_EXT_device_enumeration
    _eglExt("eglQueryDevicesEXT",                   "custom"),

    # EGL_EXT_device_query
    _eglExt("eglQueryDisplayAttribEXT",             "custom"),

    # EGL_KHR_display_reference
    _eglExt("eglQueryDisplayAttribKHR",             "custom"),

    # EGL_NV_stream_metadata
    _eglExt("eglQueryDisplayAttribNV",              "custom"),

    # EGL_KHR_debug
    _eglExt("eglDebugMessageControlKHR",            "custom"),
    _eglExt("eglQueryDebugKHR",                     "custom"),
    _eglExt("eglLabelObjectKHR",                    "custom"),
)




#!/usr/bin/env python

# (C) Copyright 2015, NVIDIA CORPORATION.
# All Rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# on the rights to use, copy, modify, merge, publish, distribute, sub
# license, and/or sell copies of the Software, and to permit persons to whom
# the Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.  IN NO EVENT SHALL
# IBM AND/OR ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
# Authors:
#    Kyle Brenneman <kbrenneman@nvidia.com>

import sys
import collections
import re
import xml.etree.ElementTree as etree

MAPI_TABLE_NUM_DYNAMIC = 4096

_LIBRARY_FEATURE_NAMES = {
    # libGL and libGLdiapatch both include every function.
    "gl" : None,
    "gldispatch" : None,
    "opengl" : frozenset(( "GL_VERSION_1_0", "GL_VERSION_1_1",
        "GL_VERSION_1_2", "GL_VERSION_1_3", "GL_VERSION_1_4", "GL_VERSION_1_5",
        "GL_VERSION_2_0", "GL_VERSION_2_1", "GL_VERSION_3_0", "GL_VERSION_3_1",
        "GL_VERSION_3_2", "GL_VERSION_3_3", "GL_VERSION_4_0", "GL_VERSION_4_1",
        "GL_VERSION_4_2", "GL_VERSION_4_3", "GL_VERSION_4_4", "GL_VERSION_4_5",
    )),
    "glesv1" : frozenset(("GL_VERSION_ES_CM_1_0", "GL_OES_point_size_array")),
    "glesv2" : frozenset(("GL_ES_VERSION_2_0", "GL_ES_VERSION_3_0",
            "GL_ES_VERSION_3_1", "GL_ES_VERSION_3_2",
    )),
}

def getFunctions(xmlFiles):
    """
    Reads an XML file and returns all of the functions defined in it.

    xmlFile should be the path to Khronos's gl.xml file. The return value is a
    sequence of FunctionDesc objects, ordered by slot number.
    """
    roots = [ etree.parse(xmlFile).getroot() for xmlFile in xmlFiles ]
    return getFunctionsFromRoots(roots)

def getFunctionsFromRoots(roots):
    functions = {}
    for root in roots:
        for func in _getFunctionList(root):
            functions[func.name] = func
    functions = functions.values()

    # Sort the function list by name.
    functions = sorted(functions, key=lambda f: f.name)

    # Assign a slot number to each function. This isn't strictly necessary,
    # since you can just look at the index in the list, but it makes it easier
    # to include the slot when formatting output.
    for i in range(len(functions)):
        functions[i] = functions[i]._replace(slot=i)

    return functions

def getExportNamesFromRoots(target, roots):
    """
    Goes through the <feature> tags from gl.xml and returns a set of OpenGL
    functions that a library should export.

    target should be one of "gl", "gldispatch", "opengl", "glesv1", or
    "glesv2".
    """
    featureNames = _LIBRARY_FEATURE_NAMES[target]
    if (featureNames == None):
        return set(func.name for func in getFunctionsFromRoots(roots))

    names = set()
    for root in roots:
        features = []
        for featElem in root.findall("feature"):
            if (featElem.get("name") in featureNames):
                features.append(featElem)
        for featElem in root.findall("extensions/extension"):
            if (featElem.get("name") in featureNames):
                features.append(featElem)
        for featElem in features:
            for commandElem in featElem.findall("require/command"):
                names.add(commandElem.get("name"))
    return names

class FunctionArg(collections.namedtuple("FunctionArg", "type name")):
    @property
    def dec(self):
        """
        Returns a "TYPE NAME" string, suitable for a function prototype.
        """
        rv = str(self.type)
        if(not rv.endswith("*")):
            rv += " "
        rv += self.name
        return rv

class FunctionDesc(collections.namedtuple("FunctionDesc", "name rt args slot")):
    def hasReturn(self):
        """
        Returns true if the function returns a value.
        """
        return (self.rt != "void")

    @property
    def decArgs(self):
        """
        Returns a string with the types and names of the arguments, as you
        would use in a function declaration.
        """
        if(len(self.args) == 0):
            return "void"
        else:
            return ", ".join(arg.dec for arg in self.args)

    @property
    def callArgs(self):
        """
        Returns a string with the names of the arguments, as you would use in a
        function call.
        """
        return ", ".join(arg.name for arg in self.args)

    @property
    def basename(self):
        assert(self.name.startswith("gl"))
        return self.name[2:]

def _getFunctionList(root):
    for elem in root.findall("commands/command"):
        yield _parseCommandElem(elem)

def _parseCommandElem(elem):
    protoElem = elem.find("proto")
    (rt, name) = _parseProtoElem(protoElem)

    args = []
    for ch in elem.findall("param"):
        # <param> tags have the same format as a <proto> tag.
        args.append(FunctionArg(*_parseProtoElem(ch)))
    func = FunctionDesc(name, rt, tuple(args), slot=None)

    return func

def _parseProtoElem(elem):
    # If I just remove the tags and string the text together, I'll get valid C code.
    text = _flattenText(elem)
    text = text.strip()
    m = re.match(r"^(.+)\b(\w+)(?:\s*\[\s*(\d*)\s*\])?$", text, re.S)
    if (m):
        typename = _fixupTypeName(m.group(1))
        name = m.group(2)
        if (m.group(3)):
            # HACK: glPathGlyphIndexRangeNV defines an argument like this:
            # GLuint baseAndCount[2]
            # Convert it to a pointer and hope for the best.
            typename += "*"
        return (typename, name)
    else:
        raise ValueError("Can't parse element %r -> %r" % (elem, text))

def _flattenText(elem):
    """
    Returns the text in an element and all child elements, with the tags
    removed.
    """
    text = ""
    if(elem.text != None):
        text = elem.text
    for ch in elem:
        text += _flattenText(ch)
        if(ch.tail != None):
            text += ch.tail
    return text

def _fixupTypeName(typeName):
    """
    Converts a typename into a more consistent format.
    """

    rv = typeName.strip()

    # Replace "GLvoid" with just plain "void".
    rv = re.sub(r"\bGLvoid\b", "void", rv)

    # Remove the vendor suffixes from types that have a suffix-less version.
    rv = re.sub(r"\b(GLhalf|GLintptr|GLsizeiptr|GLint64|GLuint64)(?:ARB|EXT|NV|ATI)\b", r"\1", rv)

    rv = re.sub(r"\bGLvoid\b", "void", rv)

    # Clear out any leading and trailing whitespace.
    rv = rv.strip()

    # Remove any whitespace before a '*'
    rv = re.sub(r"\s+\*", r"*", rv)

    # Change "foo*" to "foo *"
    rv = re.sub(r"([^\*])\*", r"\1 *", rv)

    # Condense all whitespace into a single space.
    rv = re.sub(r"\s+", " ", rv)

    return rv




#!/usr/bin/env python

"""
Generates dispatch functions for EGL.

The list of functions and arguments is read from the Khronos's XML files, with
additional information defined in the module eglFunctionList.
"""

import argparse
import collections
import sys
import textwrap

import eglFunctionList
import genCommon

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("target", choices=("header", "source"),
            help="Whether to build the source or header file.")
    parser.add_argument("xml_files", nargs="+",
            help="The XML files with the EGL function lists.")

    args = parser.parse_args()

    xmlFunctions = genCommon.getFunctions(args.xml_files)
    xmlByName = dict((f.name, f) for f in xmlFunctions)
    functions = []
    for (name, eglFunc) in eglFunctionList.EGL_FUNCTIONS:
        func = xmlByName[name]
        eglFunc = fixupEglFunc(func, eglFunc)
        functions.append((func, eglFunc))

    # Sort the function list by name.
    functions = sorted(functions, key=lambda f: f[0].name)

    if args.target == "header":
        text = generateHeader(functions)
    elif args.target == "source":
        text = generateSource(functions)
    sys.stdout.write(text)

def fixupEglFunc(func, eglFunc):
    result = dict(eglFunc)
    if result.get("prefix") is None:
        result["prefix"] = ""

    if result.get("extension") is not None:
        text = "defined(" + result["extension"] + ")"
        result["extension"] = text

    if result["method"] in ("none", "custom"):
        return result

    if result["method"] not in ("display", "device", "current"):
        raise ValueError("Invalid dispatch method %r for function %r" % (result["method"], func.name))

    if func.hasReturn():
        if result.get("retval") is None:
            result["retval"] = getDefaultReturnValue(func.rt)

    return result

def generateHeader(functions):
    text = textwrap.dedent(r"""
    #ifndef G_EGLDISPATCH_STUBS_H
    #define G_EGLDISPATCH_STUBS_H

    #ifdef __cplusplus
    extern "C" {
    #endif

    #include <EGL/egl.h>
    #include <EGL/eglext.h>
    #include "glvnd/libeglabi.h"

    """.lstrip("\n"))

    text += "enum {\n"
    for (func, eglFunc) in functions:
        text += generateGuardBegin(func, eglFunc)
        text += "    __EGL_DISPATCH_" + func.name + ",\n"
        text += generateGuardEnd(func, eglFunc)
    text += "    __EGL_DISPATCH_COUNT\n"
    text += "};\n"

    for (func, eglFunc) in functions:
        if eglFunc["inheader"]:
            text += generateGuardBegin(func, eglFunc)
            text += "{f.rt} EGLAPIENTRY {ex[prefix]}{f.name}({f.decArgs});\n".format(f=func, ex=eglFunc)
            text += generateGuardEnd(func, eglFunc)

    text += textwrap.dedent(r"""
    #ifdef __cplusplus
    }
    #endif
    #endif // G_EGLDISPATCH_STUBS_H
    """)
    return text

def generateSource(functions):
    # First, sort the function list by name.
    text = ""
    text += '#include "egldispatchstubs.h"\n'
    text += '#include "g_egldispatchstubs.h"\n'
    text += '#include <stddef.h>\n'
    text += "\n"

    for (func, eglFunc) in functions:
        if eglFunc["method"] not in ("custom", "none"):
            text += generateGuardBegin(func, eglFunc)
            text += generateDispatchFunc(func, eglFunc)
            text += generateGuardEnd(func, eglFunc)

    text += "\n"
    text += "const char * const __EGL_DISPATCH_FUNC_NAMES[__EGL_DISPATCH_COUNT + 1] = {\n"
    for (func, eglFunc) in functions:
        text += generateGuardBegin(func, eglFunc)
        text += '    "' + func.name + '",\n'
        text += generateGuardEnd(func, eglFunc)
    text += "    NULL\n"
    text += "};\n"

    text += "const __eglMustCastToProperFunctionPointerType __EGL_DISPATCH_FUNCS[__EGL_DISPATCH_COUNT + 1] = {\n"
    for (func, eglFunc) in functions:
        text += generateGuardBegin(func, eglFunc)
        if eglFunc["method"] != "none":
            text += "    (__eglMustCastToProperFunctionPointerType) " + eglFunc.get("prefix", "") + func.name + ",\n"
        else:
            text += "    NULL, // " + func.name + "\n"
        text += generateGuardEnd(func, eglFunc)
    text += "    NULL\n"
    text += "};\n"

    return text

def generateGuardBegin(func, eglFunc):
    ext = eglFunc.get("extension")
    if ext is not None:
        return "#if " + ext + "\n"
    else:
        return ""

def generateGuardEnd(func, eglFunc):
    if eglFunc.get("extension") is not None:
        return "#endif\n"
    else:
        return ""

def generateDispatchFunc(func, eglFunc):
    text = ""

    if eglFunc.get("static"):
        text += "static "
    elif eglFunc.get("public"):
        text += "PUBLIC "
    text += textwrap.dedent(
    r"""
    {f.rt} EGLAPIENTRY {ef[prefix]}{f.name}({f.decArgs})
    {{
        typedef {f.rt} EGLAPIENTRY (* _pfn_{f.name})({f.decArgs});
    """).lstrip("\n").format(f=func, ef=eglFunc)

    if func.hasReturn():
        text += "    {f.rt} _ret = {ef[retval]};\n".format(f=func, ef=eglFunc)

    text += "    _pfn_{f.name} _ptr_{f.name} = (_pfn_{f.name}) ".format(f=func)
    if eglFunc["method"] == "current":
        text += "__eglDispatchFetchByCurrent(__EGL_DISPATCH_{f.name});\n".format(f=func)

    elif eglFunc["method"] in ("display", "device"):
        if eglFunc["method"] == "display":
            lookupFunc = "__eglDispatchFetchByDisplay"
            lookupType = "EGLDisplay"
        else:
            assert eglFunc["method"] == "device"
            lookupFunc = "__eglDispatchFetchByDevice"
            lookupType = "EGLDeviceEXT"

        lookupArg = None
        for arg in func.args:
            if arg.type == lookupType:
                lookupArg = arg.name
                break
        if lookupArg is None:
            raise ValueError("Can't find %s argument for function %s" % (lookupType, func.name,))

        text += "{lookupFunc}({lookupArg}, __EGL_DISPATCH_{f.name});\n".format(
                f=func, lookupFunc=lookupFunc, lookupArg=lookupArg)
    else:
        raise ValueError("Unknown dispatch method: %r" % (eglFunc["method"],))

    text += "    if(_ptr_{f.name} != NULL) {{\n".format(f=func)
    text += "        "
    if func.hasReturn():
        text += "_ret = "
    text += "_ptr_{f.name}({f.callArgs});\n".format(f=func)
    text += "    }\n"

    if func.hasReturn():
        text += "    return _ret;\n"
    text += "}\n"
    return text

def getDefaultReturnValue(typename):
    if typename.endswith("*"):
        return "NULL"
    elif typename == "EGLDisplay":
        return "EGL_NO_DISPLAY"
    elif typename == "EGLContext":
        return "EGL_NO_CONTEXT"
    elif typename == "EGLSurface":
        return "EGL_NO_SURFACE"
    elif typename == "EGLBoolean":
        return "EGL_FALSE";

    return "0"

if __name__ == "__main__":
    main()




#!/usr/bin/env python3

import argparse
import os
import platform
import subprocess

# This list contains symbols that _might_ be exported for some platforms
PLATFORM_SYMBOLS = [
    '__bss_end__',
    '__bss_start__',
    '__bss_start',
    '__cxa_guard_abort',
    '__cxa_guard_acquire',
    '__cxa_guard_release',
    '__end__',
    '__odr_asan._glapi_Context',
    '__odr_asan._glapi_Dispatch',
    '_bss_end__',
    '_edata',
    '_end',
    '_fini',
    '_init',
    '_fbss',
    '_fdata',
    '_ftext',
]

def get_symbols_nm(nm, lib):
    '''
    List all the (non platform-specific) symbols exported by the library
    using `nm`
    '''
    symbols = []
    platform_name = platform.system()
    output = subprocess.check_output([nm, '-gP', lib],
                                     stderr=open(os.devnull, 'w')).decode("ascii")
    for line in output.splitlines():
        fields = line.split()
        if len(fields) == 2 or fields[1] == 'U':
            continue
        symbol_name = fields[0]
        if platform_name == 'Linux':
            if symbol_name in PLATFORM_SYMBOLS:
                continue
        elif platform_name == 'Darwin':
            assert symbol_name[0] == '_'
            symbol_name = symbol_name[1:]
        symbols.append(symbol_name)
    return symbols


def get_symbols_dumpbin(dumpbin, lib):
    '''
    List all the (non platform-specific) symbols exported by the library
    using `dumpbin`
    '''
    symbols = []
    output = subprocess.check_output([dumpbin, '/exports', lib],
                                     stderr=open(os.devnull, 'w')).decode("ascii")
    for line in output.splitlines():
        fields = line.split()
        # The lines with the symbols are made of at least 4 columns; see details below
        if len(fields) < 4:
            continue
        try:
            # Making sure the first 3 columns are a dec counter, a hex counter
            # and a hex address
            _ = int(fields[0], 10)
            _ = int(fields[1], 16)
            _ = int(fields[2], 16)
        except ValueError:
            continue
        symbol_name = fields[3]
        # De-mangle symbols
        if symbol_name[0] == '_':
            symbol_name = symbol_name[1:].split('@')[0]
        symbols.append(symbol_name)
    return symbols


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--symbols-file',
                        action='store',
                        required=True,
                        help='path to file containing symbols')
    parser.add_argument('--lib',
                        action='store',
                        required=True,
                        help='path to library')
    parser.add_argument('--nm',
                        action='store',
                        help='path to binary (or name in $PATH)')
    parser.add_argument('--dumpbin',
                        action='store',
                        help='path to binary (or name in $PATH)')
    parser.add_argument('--ignore-symbol',
                        action='append',
                        help='do not process this symbol')
    args = parser.parse_args()

    try:
        if platform.system() == 'Windows':
            if not args.dumpbin:
                parser.error('--dumpbin is mandatory')
            lib_symbols = get_symbols_dumpbin(args.dumpbin, args.lib)
        else:
            if not args.nm:
                parser.error('--nm is mandatory')
            lib_symbols = get_symbols_nm(args.nm, args.lib)
    except:
        # We can't run this test, but we haven't technically failed it either
        # Return the GNU "skip" error code
        exit(77)
    mandatory_symbols = []
    optional_symbols = []
    with open(args.symbols_file) as symbols_file:
        qualifier_optional = '(optional)'
        for line in symbols_file.readlines():

            # Strip comments
            line = line.split('#')[0]
            line = line.strip()
            if not line:
                continue

            # Line format:
            # [qualifier] symbol
            qualifier = None
            symbol = None

            fields = line.split()
            if len(fields) == 1:
                symbol = fields[0]
            elif len(fields) == 2:
                qualifier = fields[0]
                symbol = fields[1]
            else:
                print(args.symbols_file + ': invalid format: ' + line)
                exit(1)

            # The only supported qualifier is 'optional', which means the
            # symbol doesn't have to be exported by the library
            if qualifier and not qualifier == qualifier_optional:
                print(args.symbols_file + ': invalid qualifier: ' + qualifier)
                exit(1)

            if qualifier == qualifier_optional:
                optional_symbols.append(symbol)
            else:
                mandatory_symbols.append(symbol)

    unknown_symbols = []
    for symbol in lib_symbols:
        if symbol in mandatory_symbols:
            continue
        if symbol in optional_symbols:
            continue
        if args.ignore_symbol and symbol in args.ignore_symbol:
            continue
        if symbol[:2] == '_Z':
            # As ajax found out, the compiler intentionally exports symbols
            # that we explicitely asked it not to export, and we can't do
            # anything about it:
            # https://gcc.gnu.org/bugzilla/show_bug.cgi?id=36022#c4
            continue
        unknown_symbols.append(symbol)

    missing_symbols = [
        sym for sym in mandatory_symbols if sym not in lib_symbols
    ]

    for symbol in unknown_symbols:
        print(args.lib + ': unknown symbol exported: ' + symbol)

    for symbol in missing_symbols:
        print(args.lib + ': missing symbol: ' + symbol)

    if unknown_symbols or missing_symbols:
        exit(1)
    exit(0)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Copyright (C) 2010 LunarG Inc.
# (C) Copyright 2015, NVIDIA CORPORATION.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
#
# Authors:
#    Kyle Brenneman <kbrenneman@nvidia.com>
#
# Based on code ogiginally by:
#    Chia-I Wu <olv@lunarg.com>


"""
Generates the glapi_mapi_tmp.h header file from Khronos's XML file.
"""

import sys
import xml.etree.ElementTree as etree

import genCommon

def _main():
    target = sys.argv[1]
    xmlFiles = sys.argv[2:]

    roots = [ etree.parse(filename).getroot() for filename in xmlFiles ]
    allFunctions = genCommon.getFunctionsFromRoots(roots)

    names = genCommon.getExportNamesFromRoots(target, roots)
    functions = [f for f in allFunctions if(f.name in names)]

    if (target in ("gl", "gldispatch")):
        assert(len(functions) == len(allFunctions))
        assert(all(functions[i] == allFunctions[i] for i in range(len(functions))))
        assert(all(functions[i].slot == i for i in range(len(functions))))

    print(r"""
/* This file is automatically generated by gen_gldispatch_mapi.py.  Do not modify. */

#ifndef _GLAPI_TMP_H_
#define _GLAPI_TMP_H_
typedef int GLclampx;
typedef void (APIENTRY  *GLDEBUGPROCKHR)(GLenum source,GLenum type,GLuint id,GLenum severity,GLsizei length,const GLchar *message,const void *userParam);
#endif /* _GLAPI_TMP_H_ */
""".lstrip("\n"))

    print(generate_defines(functions))
    print(generate_table(functions, allFunctions))
    print(generate_noop_array(functions))
    print(generate_public_stubs(functions))
    print(generate_public_entries(functions))
    print(generate_stub_asm_gcc(functions, (target == "gldispatch")))

def generate_defines(functions):
    text = r"""
#ifdef MAPI_TMP_DEFINES
#define GL_GLEXT_PROTOTYPES
#include "GL/gl.h"
#include "GL/glext.h"

""".lstrip("\n")
    for func in functions:
        text += "GLAPI {f.rt} APIENTRY {f.name}({f.decArgs});\n".format(f=func)
    text += "#undef MAPI_TMP_DEFINES\n"
    text += "#endif /* MAPI_TMP_DEFINES */\n"
    return text

def generate_table(functions, allFunctions):
    text = "#ifdef MAPI_TMP_TABLE\n"
    text += "#define MAPI_TABLE_NUM_STATIC %d\n" % (len(allFunctions))
    text += "#define MAPI_TABLE_NUM_DYNAMIC %d\n" % (genCommon.MAPI_TABLE_NUM_DYNAMIC,)
    text += "#undef MAPI_TMP_TABLE\n"
    text += "#endif /* MAPI_TMP_TABLE */\n"
    return text

def generate_noop_array(functions):
    text = "#ifdef MAPI_TMP_NOOP_ARRAY\n"
    text += "#ifdef DEBUG\n\n"

    for func in functions:
        text += "static {f.rt} APIENTRY noop{f.basename}({f.decArgs})\n".format(f=func)
        text += "{\n"
        if (len(func.args) > 0):
            text += "  "
            for arg in func.args:
                text += " (void) {a.name};".format(a=arg)
            text += "\n"
        text += "   noop_warn(\"{f.name}\");\n".format(f=func)
        if (func.hasReturn()):
            text += "   return ({f.rt}) 0;\n".format(f=func)
        text += "}\n\n"

    text += "const mapi_func table_noop_array[] = {\n"
    for func in functions:
        text += "   (mapi_func) noop{f.basename},\n".format(f=func)
    for i in range(genCommon.MAPI_TABLE_NUM_DYNAMIC - 1):
        text += "   (mapi_func) noop_generic,\n"
    text += "   (mapi_func) noop_generic\n"
    text += "};\n\n"
    text += "#else /* DEBUG */\n\n"
    text += "const mapi_func table_noop_array[] = {\n"
    for i in range(len(functions) + genCommon.MAPI_TABLE_NUM_DYNAMIC - 1):
        text += "   (mapi_func) noop_generic,\n"
    text += "   (mapi_func) noop_generic\n"

    text += "};\n\n"
    text += "#endif /* DEBUG */\n"
    text += "#undef MAPI_TMP_NOOP_ARRAY\n"
    text += "#endif /* MAPI_TMP_NOOP_ARRAY */\n"
    return text

def generate_public_stubs(functions):
    text = "#ifdef MAPI_TMP_PUBLIC_STUBS\n"

    text += "static const struct mapi_stub public_stubs[] = {\n"
    for func in functions:
        text += "   { \"%s\", %d },\n" % (func.name, func.slot)
    text += "};\n"
    text += "#undef MAPI_TMP_PUBLIC_STUBS\n"
    text += "#endif /* MAPI_TMP_PUBLIC_STUBS */\n"
    return text

def generate_public_entries(functions):
    text = "#ifdef MAPI_TMP_PUBLIC_ENTRIES\n"

    for func in functions:
        retStr = ("return " if func.hasReturn() else "")
        text += r"""
GLAPI {f.rt} APIENTRY {f.name}({f.decArgs})
{{
   const struct _glapi_table *_tbl = entry_current_get();
   mapi_func _func = ((const mapi_func *) _tbl)[{f.slot}];
   {retStr}(({f.rt} (APIENTRY *)({f.decArgs})) _func)({f.callArgs});
}}

""".lstrip("\n").format(f=func, retStr=retStr)

    text += "\n"
    text += "static const mapi_func public_entries[] = {\n"
    for func in functions:
        text += "   (mapi_func) %s,\n" % (func.name,)
    text += "};\n"
    text += "#undef MAPI_TMP_PUBLIC_ENTRIES\n"
    text += "#endif /* MAPI_TMP_PUBLIC_ENTRIES */\n"
    return text

def generate_stub_asm_gcc(functions, includeDynamic):
    text = "#ifdef MAPI_TMP_STUB_ASM_GCC\n"
    text += "__asm__(\n"

    for func in functions:
        text += 'STUB_ASM_ENTRY("%s")"\\n"\n' % (func.name,)
        text += '"\\t"STUB_ASM_CODE("%d")"\\n"\n\n' % (func.slot,)

    if (includeDynamic):
        for i in range(genCommon.MAPI_TABLE_NUM_DYNAMIC):
            text += 'STUB_ASM_ENTRY("dynamic_%04d")"\\n"\n' % (i,)
            text += '"\\t"STUB_ASM_CODE("%d")"\\n"\n\n' % (len(functions) + i)
    text += ");\n"
    text += "#undef MAPI_TMP_STUB_ASM_GCC\n"
    text += "#endif /* MAPI_TMP_STUB_ASM_GCC */\n"
    return text

if (__name__ == "__main__"):
    _main()




#!/usr/bin/env python

# (C) Copyright 2015, NVIDIA CORPORATION.
# All Rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# on the rights to use, copy, modify, merge, publish, distribute, sub
# license, and/or sell copies of the Software, and to permit persons to whom
# the Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.  IN NO EVENT SHALL
# IBM AND/OR ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
# Authors:
#    Kyle Brenneman <kbrenneman@nvidia.com>

"""
Generates the list of functions that should be exported from libOpenGL.so.
"""

import sys
import xml.etree.ElementTree as etree

import genCommon

def _main():
    target = sys.argv[1]
    xmlFiles = sys.argv[2:]
    roots = [ etree.parse(filename).getroot() for filename in xmlFiles ]

    names = genCommon.getExportNamesFromRoots(target, roots)
    for name in sorted(names):
        print(name)

if (__name__ == "__main__"):
    _main()

